from zope.interface import Interface

from Products.CMFPlone.interfaces.Translatable import ITranslatable

class ILanguageTool(Interface):
    """Marker interface for the portal_languages tool.
    """
